unit ClipboardHistory;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, ExtCtrls, ClipBrd, LazUTF8, LCLType;

type
  TClipHistory = class
  public

    constructor Create;
    procedure MakeList;
    procedure MakeSQLList;
  end;

var
  ClipHistory: TClipHistory;
  Timer: TTimer;

implementation

procedure StripStrings(t: TStringList);
var i: integer;
begin
  i:=0;
  while i < t.Count do
    if UTF8Trim(t[i]) = '' then
      t.Delete(i)
    else begin
      t[i] := UTF8Trim(t[i]);
      i:=i+1;
    end;

end;

function ContainsStrings(t: TStringList): boolean;
var
  i, j: integer;
begin
  for i:=0 to t.Count-1 do begin
    for j:=1 to UTF8Length(t[i]) do begin
       if not(UTF8Copy(t[i], j, 1)[1] in ['0'..'9', '.']) or (UTF8Length(t.Strings[i]) >= 20) then
         exit(true);
    end;
  end;

  exit(false);
end;

constructor TClipHistory.Create;
begin
end;

procedure TClipHistory.MakeList;
var
  items: TStringList;
  i: integer;
begin
  try
    items := TStringList.Create();
    items.Text := Clipboard.AsText;
    StripStrings(items);
    items.Delimiter := ',';

    if ContainsStrings(items) then begin
      for i:=0 to items.Count-1 do begin
        items[i] := '''' + items[i] + '''';
      end;
    end;

    i:=0;
    while i < items.Count do
      if UTF8Trim(items[i]) = '' then
        items.Delete(i)
      else
        i:=i+1;

    Clipboard.AsText := items.DelimitedText;
  finally
    items.Free;
  end;
end;

procedure TClipHistory.MakeSQLList;
var
  Text: TStringList;
  i, n: integer;
  isIntList: boolean;
begin
  try
    Text := TStringList.Create;
    StripStrings(Text);

    if Text.Count < 1 then
      exit;

    isIntList := True;
    for i := 0 to Text.Count - 1 do
      if (UTF8Length(Text[i]) > 9) or (not TryStrToInt(Text[i], n)) then begin
        isIntList := False;
        break;
      end;
    if isIntList then
      Clipboard.AsText := 'SELECT ' + UTF8StringReplace(UTF8Trim(Text.Text), #13#10, ' UNION'#13#10'SELECT ', [rfReplaceAll]) + ' FROM DUAL'
    else
      Clipboard.AsText := 'SELECT ''' + UTF8StringReplace(UTF8Trim(Text.Text), #13#10, ''' UNION'#13#10'SELECT ''', [rfReplaceAll]) + ''' FROM DUAL';
  finally
    Text.Free;
  end;
end;

begin
  ClipHistory := TClipHistory.Create;
end.
