unit SystemLoad;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, ExtCtrls, Graphics, Forms, Dialogs;

var
  FLastIdleTime: int64;
  FLastKernelTime: int64;
  FLastUserTime: int64;
  image: TBitmap;

procedure UpdateTrayIcon(TrayIcon: TTrayIcon);
procedure SetDPI(dpi: integer);

implementation

uses jwaWinBase;

function GetCPULoadPercent: integer;
var
  IdleTimeRec: TFileTime;
  KernelTimeRec: TFileTime;
  UserTimeRec: TFileTime;
  IdleTime: int64 absolute IdleTimeRec;
  KernelTime: int64 absolute KernelTimeRec;
  UserTime: int64 absolute UserTimeRec;
  IdleDiff: int64;
  KernelDiff: int64;
  UserDiff: int64;
  SysTime: int64;
begin
  if GetSystemTimes(@IdleTimeRec, @KernelTimeRec, @UserTimeRec) then
  begin
    IdleDiff := IdleTime - FLastIdleTime;
    KernelDiff := KernelTime - FLastKernelTime;
    UserDiff := UserTime - FLastUserTime;
    FLastIdleTime := IdleTime;
    FLastKernelTime := KernelTime;
    FLastUserTime := UserTime;
    SysTime := KernelDiff + UserDiff;
    Result := 100 * (SysTime - IdleDiff) div SysTime;
  end
  else
    Result := 0;
end;

function DarkColor(color: TColor): TColor;
var
  r, g, b: byte;
begin
  RedGreenBlue(color, r, g, b);
  r := r div 4;
  g := g div 4;
  b := b div 4;
  Result := RGBToColor(r, g, b);
end;

function FillColor(percent: integer): TColor;
begin
  if percent < 50 then
    Result := RGBToColor(trunc((50 - percent) / 50 * 255), 255,
      trunc((50 - percent) / 50 * 255))
  else if percent < 75 then
    Result := RGBToColor(trunc((percent - 50) / 25 * 255), 255, 0)
  else if percent < 100 then
    Result := RGBToColor(255, trunc((100 - percent) / 25 * 255), 0)
  else
    Result := RGBToColor(255, 0, 0);
end;

function GetMemoryLoadPercent: integer;
var
  m: TMemoryStatus;
  usedMem: QWord;
begin
  GlobalMemoryStatus(m);

  Result := (m.dwTotalPageFile - m.dwAvailPageFile) * 100 div m.dwTotalPhys;

  // Если вышли за RAM, то 200% будет пиком
  usedMem := m.dwTotalPageFile - m.dwAvailPageFile;
  if Result > 100 then
    Result := 100 + (usedMem - m.dwTotalPhys) * 100 div (m.dwTotalPageFile-m.dwTotalPhys);
end;

function GetTimePercent: integer;
var
  h, m, s, ms: word;
  minSinceDayBegin: integer;
begin
  DecodeTime(Time, h, m, s, ms);
  minSinceDayBegin := h * 60 + m;
  Result := (minSinceDayBegin - 510) * 100 div (1050 - 510);
end;

procedure DrawLine(const aColor: TColor; const aLineNum, aTotalLineNum, spaceSize, aValue: integer);
begin
  image.Canvas.Brush.Color := DarkColor(aColor);
  image.Canvas.FillRect(0,                                   image.Canvas.Height * (aLineNum-1) div aTotalLineNum,
                        aValue * image.Canvas.Width div 100, image.Canvas.Height div aTotalLineNum * aLineNum - spaceSize);

  image.Canvas.Brush.Color := aColor;
  image.Canvas.FillRect(aValue * image.Canvas.Width div 100-1, image.Canvas.Height * (aLineNum-1) div aTotalLineNum,
                        aValue * image.Canvas.Width div 100,     image.Canvas.Height div aTotalLineNum * aLineNum - spaceSize);
end;

procedure DrawLine2(const aColor: TColor; const aLineNum, aTotalLineNum, spaceSize, aValue: integer);
begin
  if aValue <= 100 then begin
    DrawLine(aColor, aLineNum, aTotalLineNum, spaceSize, aValue);
    exit;
  end;

  image.Canvas.Brush.Color := DarkColor(aColor);
  image.Canvas.FillRect(0,                  image.Canvas.Height * (aLineNum-1) div aTotalLineNum,
                        image.Canvas.Width, image.Canvas.Height div aTotalLineNum * aLineNum - spaceSize);

  image.Canvas.Brush.Color := aColor;
  image.Canvas.FillRect(0,                                           image.Canvas.Height * (aLineNum-1) div aTotalLineNum,
                        aValue mod 100 * image.Canvas.Width div 100+1, image.Canvas.Height div aTotalLineNum * aLineNum - spaceSize);
end;

procedure DrawLine3(const aColor: TColor; const aLineNum, aTotalLineNum, spaceSize, aValue: integer);
var
  i: integer;
begin
  image.Canvas.Brush.Color := DarkColor(aColor);
  for i := 0 to aValue * image.Canvas.Width div 100 do begin
    if i mod 2 = 0 then
      image.Canvas.FillRect(i, image.Canvas.Height * (aLineNum-1) div aTotalLineNum, i+1, image.Canvas.Height div aTotalLineNum * aLineNum - spaceSize);
  end;

  image.Canvas.Brush.Color := aColor;
  for i := (aValue * image.Canvas.Width div 100)-2 to aValue * image.Canvas.Width div 100 do begin
    if i mod 2 = 0 then
      image.Canvas.FillRect(i, image.Canvas.Height * (aLineNum-1) div aTotalLineNum, i+1, image.Canvas.Height div aTotalLineNum * aLineNum - spaceSize);
  end;
end;

procedure UpdateTrayIcon(TrayIcon: TTrayIcon);
const
  lineCount: integer = 2;
  space: integer = 1;
var
  p, m{, t}: integer; // CPU usage, MEM usage, time percent
begin
  p := GetCPULoadPercent;
  m := GetMemoryLoadPercent;
  // t := GetTimePercent;

  //TrayIcon.Hint := format('CPU: %d%%'#10'MEM: %d%%'#10'TIM: %d%%', [p, m, t]);
  TrayIcon.Hint := format('CPU: %d%%'#10'MEM: %d%%', [p, m]);

  image.Canvas.Brush.Color := clBlack {RGBToColor(38, 37, 36)};
  image.Canvas.Clear;

  DrawLine2(FillColor(p), 1, lineCount, space, p);
  DrawLine2(FillColor(m), 2, lineCount, space, m);
  //DrawLine2(FillColor(t), 3, lineCount, space, t);

  TrayIcon.Icon.Assign(image);
end;

procedure SetDPI(dpi: integer);
begin
  image.Width := trunc(double(16) * double(dpi) / double(96));
  image.Height := trunc(double(16) * double(dpi) / double(96));
end;

begin
  image := TBitmap.Create;
  image.Width := 16;
  image.Height := 16;
end.

